function cDetect(image, w, t, k, r, ct)
    
    thresh = t;   % Harris corner threshold
    K = k;
    windowRadius = w;
    
    nonmaxrad = r;  % Non-maximal suppression radius
    
    im = image;
    if size(im,3)==3
        im=rgb2gray(im); % Transform RGB image to a Gray one. 
    end
    
    if ~isa(im,'double')
        im = double(im);
    end
   
    sigmaForWeight = 385;
    [cim0, r0, c0] = NLharris_1Normal(im,thresh, nonmaxrad,K,windowRadius,0,sigmaForWeight,0);
    
    response = [];
    for i=1:length(r0)
        response(i,:) = [r0(i) c0(i) cim0(r0(i),c0(i))];
    end
    
    zerosfill = zeros(length(r0),5);
    response = [response zerosfill];
    %here we have got the potential positions 
    smoothScale = [0 0.4 0.8 1.0 1.2 1.4 1.6 1.8 2.0];
    
    for scaleIndex = 1:length(smoothScale)
        [cimscale] = ScaleFilter(im,thresh, nonmaxrad,K,windowRadius,0,sigmaForWeight,smoothScale(scaleIndex));
       
        for i=1:length(r0)
              currentR = max(max(cimscale(response(i,1)-2:response(i,1)+2,response(i,2)-2:response(i,2)+2)));
              if scaleIndex == 1
                  if currentR < 0
                     response(i,3) = 0;
                  else
                     response(i,3) = currentR;
                  end
              else
                  if currentR < 0
                     response(i,scaleIndex+2) = 0;
                  else
                     response(i,scaleIndex+2) = currentR / response(i,3);
                  end
              end
        end
    end
    
    for i=length(r0):-1:1
        if sum(response(i,4:2+length(smoothScale))) / length(smoothScale) < ct
            response(i,:) = [];
        end
    end

    houseNLHarris = [response(:,1) response(:,2)];
    im = image;
    marked_img = im;
    for i=1:size(houseNLHarris)
        marked_img=mark(marked_img,houseNLHarris(i,1),houseNLHarris(i,2),5);
    end
    
    figure;
    imshow(uint8(marked_img));
end

function [cim, r, c] = NLharris_1Normal(im, thresh, NonMaximalradius,K, searchWinRadius, neighborWinRadius,sigmaForWeight,smoothscale)

    if ~isa(im,'double')
        im = double(im);
    end
    if smoothscale ~= 0
        gsmall = fspecial('gaussian',max(1,ceil(6*smoothscale)), smoothscale);
        im = conv2(im, gsmall, 'same'); 
    end
    dx = [1 0 -1; 1 0 -1; 1 0 -1];   % Derivative masks
    dy = [-1 -1 -1; 0 0 0; 1 1 1];
    
    Ix = conv2(im, dx, 'same');      % Image derivatives
    Iy = conv2(im, dy, 'same');    

    Ix2 = Ix.^2;
    Iy2 = Iy.^2;
    Ixy = Ix.*Iy;
    %diffVector = [];
    [Ix2,Iy2,Ixy] = NLmeansfilter_1normal(Ix,Iy,Ix2,Iy2,Ixy,searchWinRadius,neighborWinRadius,sigmaForWeight);
    %[Ix2,Iy2,Ixy] = NLmeansfilter_SquareDist(Ix,Iy,Ix2,Iy2,Ixy,searchWinRadius,neighborWinRadius,sigmaForWeight);
    %cim = (Ix2.*Iy2 - Ixy.^2)./(Ix2 + Iy2 + eps); % My preferred  measure.
    %cim = (Ix2.*Iy2 - Ixy.^2) - K*(Ix2 + Iy2).^2; % Original Harris measure.  
    cim = (Ix2.*Iy2 - Ixy.^2);
    %cim = Ix2 + Iy2 - sqrt((Ix2-Iy2).^2 + 4*Ixy.^2);
    [r,c] = nonmaxsuppts(cim, NonMaximalradius, thresh);		
end

function [outputIx2,outputIy2,outputIxy] = NLmeansfilter_1normal(Ix,Iy,Ix2,Iy2,Ixy,searchWinRadius,neighborRadius,sigmaForWeight)
%it use sum of absolute difference of the partial derivatives
% Size of the image
 [m n]=size(Ix);
 % Memory for the output
 outputIx2 = zeros(m,n);
 outputIy2 = zeros(m,n);
 outputIxy = zeros(m,n);
 % Replicate the boundaries of the input image
 oriIx = padarray(Ix,[neighborRadius neighborRadius],'symmetric');
 oriIy = padarray(Iy,[neighborRadius neighborRadius],'symmetric');
 
 Ix22 = padarray(Ix2,[neighborRadius neighborRadius],'symmetric');
 Iy22 = padarray(Iy2,[neighborRadius neighborRadius],'symmetric');
 Ixy2 = padarray(Ixy,[neighborRadius neighborRadius],'symmetric');
 sigmaForGeoDist = (2*searchWinRadius+1)/3;
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 diffMap  = zeros(m,n);
 
 for rowOfOutput = 1:m
     for colOfOutput =1:n
         rowOfCurrentPixel = rowOfOutput + neighborRadius;
         colOfCurrentPixel = colOfOutput + neighborRadius;

         CurrentPixelInIx = oriIx(rowOfCurrentPixel,colOfCurrentPixel);
         CurrentPixelInIy = oriIy(rowOfCurrentPixel,colOfCurrentPixel);
          
         dmin = 6553500; 
         averageIx2 = 0;
         averageIy2 = 0;
         averageIxy = 0;
         sweight = 0;
         
         %rmin rmax cmin cmax are the boundaries of the search window.
         rmin = max(rowOfCurrentPixel-searchWinRadius,neighborRadius+1);
         rmax = min(rowOfCurrentPixel+searchWinRadius,m+neighborRadius);
         cmin = max(colOfCurrentPixel-searchWinRadius,neighborRadius+1);
         cmax = min(colOfCurrentPixel+searchWinRadius,n+neighborRadius);
         
         
         for rowInSearchWin = rmin:1:rmax
             for colInSearchWin = cmin:1:cmax
                 %if(rowInSearchWin == rowOfCurrentPixel && colInSearchWin == colOfCurrentPixel)
                 %For the current pixel, need special process
                 %   continue; 
                 %end
                 testInIx = oriIx(rowInSearchWin, colInSearchWin);                
                 testInIy = oriIy(rowInSearchWin, colInSearchWin);                

                 %dInIx = sum(sum(kernel.*(neighborOfCurrentPixelInIx-testNeighborInIx).*(neighborOfCurrentPixelInIx-testNeighborInIx)));                
                 %dInIy = sum(sum(kernel.*(neighborOfCurrentPixelInIy-testNeighborInIy).*(neighborOfCurrentPixelInIy-testNeighborInIy)));                
                 dInIx = abs(CurrentPixelInIx-testInIx);                
                 dInIy = abs(CurrentPixelInIy-testInIy);  
                 
                 d = dInIx + dInIy;
                 %d = d/1530;
                 %d = d/1170450 * 3 * sigmaForWeight ;
                 diffMap(rowInSearchWin,colInSearchWin) = d;

             end
         end
         
         %here we have got the differences with the centered pixel within the search window 
         %diffMap(rowOfCurrentPixel,colOfCurrentPixel) = dmin;
         localDiffMap = diffMap(rmin:rmax,cmin:cmax);
         %normalize to 0~3sigmaForweight
         localDiffMap = localDiffMap ./ (max(max(localDiffMap)) + eps)*1020;
         %weightMap = exp(-localDiffMap.^2 /2/sigmaForWeight^2);
         weightMap = exp(-localDiffMap.^2 /2/sigmaForWeight^2); 
         sumWeight = sum(sum(weightMap));
         averageIx2 = sum(sum(Ix22(rmin:rmax,cmin:cmax).* weightMap)) / sumWeight;
         averageIy2 = sum(sum(Iy22(rmin:rmax,cmin:cmax).* weightMap)) / sumWeight;
         averageIxy = sum(sum(Ixy2(rmin:rmax,cmin:cmax).* weightMap)) / sumWeight;
         %until now, we have got the average intensity of the search
         %window.and it is time for the process of the current pixel.
     
         if sumWeight > 0
            outputIx2(rowOfOutput,colOfOutput) = averageIx2;
            outputIy2(rowOfOutput,colOfOutput) = averageIy2;
            outputIxy(rowOfOutput,colOfOutput) = averageIxy;
         else
            outputIx2(rowOfOutput,colOfOutput) = Ix2(rowOfOutput,colOfOutput);
            outputIy2(rowOfOutput,colOfOutput) = Iy2(rowOfOutput,colOfOutput);
            outputIxy(rowOfOutput,colOfOutput) = Ixy(rowOfOutput,colOfOutput);
         end
     end
 end
end

function [r,c, rsubp, csubp] = nonmaxsuppts(cim, radius, thresh, im)
    [rows,cols] = size(cim);
    
    % Extract local maxima by performing a grey scale morphological
    % dilation and then finding points in the corner strength image that
    % match the dilated image and are also greater than the threshold.
    
    sze = 2*radius+1;                   % Size of dilation mask.
    mx = ordfilt2(cim,sze^2,ones(sze)); % Grey-scale dilate.

    % Make mask to exclude points within radius of the image boundary. 
    bordermask = zeros(size(cim));
    bordermask(radius+1:end-radius, radius+1:end-radius) = 1;
    
    % Find maxima, threshold, and apply bordermask
    cimmx = (cim==mx) & (cim>thresh) & bordermask;
    
    [labeledImage,numberOfRegions] = bwlabel(cimmx);
    for regionIndex = 1:numberOfRegions
        [rows,cols]= find(labeledImage == regionIndex);
        for index = 2: length(rows)
            cimmx(rows(index),cols(index)) = 0;
        end
    end
    [r,c] = find(cimmx);                % Find row,col coords.

end

function [cim] = ScaleFilter(im, thresh, NonMaximalradius,K, searchWinRadius, neighborWinRadius,sigmaForWeight,smoothscale)

    if ~isa(im,'double')
        im = double(im);
    end
    if smoothscale ~= 0
        gsmall = fspecial('gaussian',max(1,round(6*smoothscale)), smoothscale);
        im = conv2(im, gsmall, 'same'); 
    end
    dx = [1 0 -1; 1 0 -1; 1 0 -1];   % Derivative masks
    dy = [-1 -1 -1; 0 0 0; 1 1 1];
    
    Ix = conv2(im, dx, 'same');      % Image derivatives
    Iy = conv2(im, dy, 'same');    

    Ix2 = Ix.^2;
    Iy2 = Iy.^2;
    Ixy = Ix.*Iy;
    %diffVector = [];
    [Ix2,Iy2,Ixy] = NLmeansfilter_1normal(Ix,Iy,Ix2,Iy2,Ixy,searchWinRadius,neighborWinRadius,sigmaForWeight);
    cim = (Ix2.*Iy2 - Ixy.^2) - K*(Ix2 + Iy2).^2; % Original Harris measure.  
    %cim = (Ix2.*Iy2 - Ixy.^2);
    %cim = Ix2 + Iy2 - sqrt((Ix2-Iy2).^2 + 4*Ixy.^2);
end

function img1=mark(img,x,y,w)

    [M,N,C]=size(img);
    img1=img;

    if isa(img,'logical')
        img1(max(1,x-floor(w/2)):min(M,x+floor(w/2)),max(1,y-floor(w/2)):min(N,y+floor(w/2)),:)=...
            (img1(max(1,x-floor(w/2)):min(M,x+floor(w/2)),max(1,y-floor(w/2)):min(N,y+floor(w/2)),:)<1);
        img1(x-floor(w/2)+1:x+floor(w/2)-1,y-floor(w/2)+1:y+floor(w/2)-1,:)=...
            img(x-floor(w/2)+1:x+floor(w/2)-1,y-floor(w/2)+1:y+floor(w/2)-1,:);
    else
        img1(max(1,x-floor(w/2)):min(M,x+floor(w/2)),max(1,y-floor(w/2)):min(N,y+floor(w/2)),:)=...
            (img1(max(1,x-floor(w/2)):min(M,x+floor(w/2)),max(1,y-floor(w/2)):min(N,y+floor(w/2)),:)<128)*255;
        img1(x-floor(w/2)+1:x+floor(w/2)-1,y-floor(w/2)+1:y+floor(w/2)-1,:)=...
            img(x-floor(w/2)+1:x+floor(w/2)-1,y-floor(w/2)+1:y+floor(w/2)-1,:);
    end
end